using BusinessLogic.BusinessLogic;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;
using TransportGuideContracts.BusinessLogicsContracts;
using TransportGuideContracts.StoragesContracts;
using TransportGuideDatabaseImplements.Implements;

namespace TransportGuide
{
    internal static class Program
    {
		private static ServiceProvider? _serviceProvider;
		public static ServiceProvider? ServiceProvider => _serviceProvider;
		/// <summary>
		///  The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main()
		{
			// To customize application configuration such as set high DPI settings or default font,
			// see https://aka.ms/applicationconfiguration.
			ApplicationConfiguration.Initialize();
			var services = new ServiceCollection();
			ConfigureServices(services);
			_serviceProvider = services.BuildServiceProvider();
			Application.Run(_serviceProvider.GetRequiredService<FormMain>());
		}
		private static void ConfigureServices(ServiceCollection services)
		{
			services.AddLogging(option =>
			{
				option.SetMinimumLevel(LogLevel.Information);
				option.AddNLog("nlog.config");
			});
			services.AddTransient<IRouteStorage, RouteStorage>();
			services.AddTransient<IStopStorage, StopStorage>();
			services.AddTransient<ITransportTypeStorage, TransportTypeStorage>();

			services.AddTransient<IRouteLogic, RouteLogic>();
			services.AddTransient<IStopLogic, StopLogic>();
			services.AddTransient<ITransportTypeLogic, TransportTypeLogic>();

			services.AddTransient<FormMain>();
			services.AddTransient<FormTransportType>();
			services.AddTransient<FormTransportTypes>();
			services.AddTransient<FormStops>();
			services.AddTransient<FormStop>();
			services.AddTransient<FormRoutes>();
			services.AddTransient<FormRoute>();
			services.AddTransient<FormStopRoute>();
			services.AddTransient<FormTests>();

		}
	}
}