﻿using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TransportGuideContracts.BindingModels;
using TransportGuideContracts.SearchModels;
using TransportGuideContracts.StoragesContracts;
using TransportGuideContracts.ViewModels;
using TransportGuideDatabaseImplements.Models;

namespace TransportGuideDatabaseImplements.Implements
{
	public class TransportTypeStorage : ITransportTypeStorage
	{
		public TransportTypeViewModel? Delete(TransportTypeBindingModel model)
		{
			using var context = new TransportGuideDB();

			var element = context.TransportTypes.FirstOrDefault(rec => rec.Id == model.Id);

			if (element != null)
			{
				context.TransportTypes.Remove(element);
				context.SaveChanges();
				return element.GetViewModel;
			}

			return null;
		}

		public TransportTypeViewModel? GetElement(TransportTypeSearchModel model)
		{
			if (string.IsNullOrEmpty(model.Name) && !model.Id.HasValue)
			{
				return null;
			}

			using var context = new TransportGuideDB();

			return context.TransportTypes.FirstOrDefault(x => (!string.IsNullOrEmpty(model.Name) && x.Name == model.Name) || (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
		}

		public List<TransportTypeViewModel> GetFilteredList(TransportTypeSearchModel model)
		{
			if (string.IsNullOrEmpty(model.Name))
			{
				return new();
			}

			using var context = new TransportGuideDB();

			return context.TransportTypes.Where(x => x.Name.Contains(model.Name)).ToList().Select(x => x.GetViewModel).ToList();
		}

		public List<TransportTypeViewModel> GetFullList()
		{
			using var context = new TransportGuideDB();

			return context.TransportTypes.ToList().Select(x => x.GetViewModel).ToList();
		}

		public TransportTypeViewModel? Insert(TransportTypeBindingModel model)
		{
			using var context = new TransportGuideDB();

			var newTransportType = TransportType.Create(model);

			if (newTransportType == null)
			{
				return null;
			}

			context.TransportTypes.Add(newTransportType);
			context.SaveChanges();

			return newTransportType.GetViewModel;
		}

		public TransportTypeViewModel? Update(TransportTypeBindingModel model)
		{
			using var context = new TransportGuideDB();

			using var transaction = context.Database.BeginTransaction();

			try
			{
				var transportType = context.TransportTypes.FirstOrDefault(rec => rec.Id == model.Id);

				if (transportType == null)
				{
					return null;
				}

				transportType.Update(model);
				context.SaveChanges();
				transaction.Commit();

				return transportType.GetViewModel;
			}
			catch
			{
				transaction.Rollback();

				throw;
			}
		}
	}
}
